/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package modelos;

import interfaces.IGestorPersonas;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;

public class GestorPersonas implements IGestorPersonas {    
    private static GestorPersonas instancia; 
    
    /**
     * Constructor
    */
    private GestorPersonas() {
    }
    
    /**
     * Método estático que permite crear una única instancia de GestorPersonas
     * @return GestorPersonas
    */                                                            
    public static GestorPersonas crear() {
        if (instancia == null) 
            instancia = new GestorPersonas();
        return instancia;        
    }   
    
    /**
     * Devuelve una lista de personas cuyo apellido comience con la cadena que se especifica
     * Si la cadena es vacía, la lista contiene todas las personas creadas
     * Si no encuentra ninguna persona con esas características, devuelve una lista vacía
     * Si hay algún problema con la BD, devuelve null
     * @param cadena comienzo del apellido
     * @return List<Persona>  - lista con las personas cuyo apellido empiece con la cadena especificada
     * @see Persona
    */                                                        
    @Override
    public List<Persona> buscar(String cadena) {
        List<Persona> personas = new ArrayList<>();        
        CallableStatement cstmt = null;
        try {            
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_buscar_personas(?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setString(1, cadena);
            ResultSet rs = cstmt.executeQuery();
            while (rs.next()) {
                int idPersona = rs.getInt("IDPersona");
                String apellidos = rs.getString("Apellidos");
                String nombres = rs.getString("Nombres");
                LocalDate fechaNacimiento = null;
                if (rs.getDate("FechaNacimiento") != null)
                    fechaNacimiento = Instant.ofEpochMilli(rs.getDate("FechaNacimiento").getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
                String domicilio = rs.getString("Domicilio");
                String email = rs.getString("Email");
                Persona unaPersona = new Persona(idPersona, apellidos, nombres, fechaNacimiento, domicilio, email);
                personas.add(unaPersona);                   
            }
            rs.close();
            return personas;
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        } 
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                            
    }
        
    /**
     * Crea una Persona nueva en la base de datos
     * @param apellidos apellidos de la persona a dar de alta
     * @param nombres nombres de la persona a dar de alta
     * @param fechaNacimiento fecha de nacimiento de la persona a dar de alta
     * @param direccion dirección de la persona a dar de alta
     * @param correo correo de la persona a dar de alta
     * @return String  - cadena con el estado de la operación 
     * @see Persona
    */                                                            
    @Override
    public String altaPersona(String apellidos, String nombres, LocalDate fechaNacimiento, String direccion, String correo) {
        CallableStatement cstmt = null;
        try {
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_alta_persona(?, ?, ?, ?, ?, ?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setString(1, apellidos);
            cstmt.setString(2, nombres);
            if (fechaNacimiento != null)
                cstmt.setDate(3, java.sql.Date.valueOf(fechaNacimiento));
            else
                cstmt.setDate(3, null);
            cstmt.setString(4, direccion);
            cstmt.setString(5, correo);            
            cstmt.registerOutParameter(6, java.sql.Types.VARCHAR);            
            cstmt.executeUpdate();
            return cstmt.getString(6);
        }
        catch (Exception e) {
           return e.getMessage();
        }
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                    
    }
    
    /**
     * Borra la Persona especificada de la base de datos
     * @param unaPersona persona a borrar
     * @return String  - cadena con el estado de la operación
     * @see Persona
    */                                                                
    @Override
    public String borrarPersona(Persona unaPersona) {
        CallableStatement cstmt = null;
        try {
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_borrar_persona(?, ?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setInt(1, unaPersona.verID());
            cstmt.registerOutParameter(2, java.sql.Types.VARCHAR);
            cstmt.executeUpdate();
            return cstmt.getString(2);            
        }
        catch (Exception e) {
            return e.getMessage();
        }
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                    
    }
        
    /**
     * Devuelve una cadena con las personas que cumplen año hoy
     * @return String  - cadena con las personas que cumplen año hoy
     * @see Persona
    */                                                                    
    @Override
    public String hoyCumple() {
        StringBuilder resultado = new StringBuilder();
        CallableStatement cstmt = null;
        try {
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_hoy_cumple()}";
            cstmt = cnn.prepareCall(sql);
            ResultSet rs = cstmt.executeQuery();
            while (rs.next())
                resultado.append(rs.getString("Nombre")).append("\n");
            rs.close();
            return resultado.toString();
        }
        catch (Exception e) {
            return e.getMessage();
        }     
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                    
    }
    
    /**
     * Devuelve una lista con el calendario de cumpleaños de todas las personas
     * La lista tiene la forma: dia/mes/nombre
     * Si no hay personas 
     * Si se produce algún error, devuelve null
     * @return List<String>  - lista con el calendario de cumpleaños de todas las personas
    */                                                                        
    @Override
    public List<String> calendario() {
        CallableStatement cstmt = null;
        try {
            List<String> calendario = new ArrayList<>();
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_calendario()}";
            cstmt = cnn.prepareCall(sql);
            ResultSet rs = cstmt.executeQuery();
            StringBuilder entradaCalendario;
            while (rs.next()) {
                entradaCalendario = new StringBuilder();
                if (rs.getInt("Dia") != 0) {
                    //si la fecha es nula, rs.getInt() devuelve 0
                    int dia = rs.getInt("Dia");
                    int mes = rs.getInt("Mes");
                    String nombre = rs.getString("Nombre");
                    entradaCalendario.append(Integer.toString(dia)).append("/").append(Integer.toString(mes)).append("/").append(nombre);
                    calendario.add(entradaCalendario.toString());                   
                }
            }
            rs.close();
            return calendario;
        }
        catch(Exception e) {
            return null;
        }           
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }                    
    }
}
