package modelos;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

public class Persona {
    public static final String MENSAJE_TELEFONO_ERROR_CONEXION = "Error de conexión";
    public static final String MENSAJE_ALTA_TELEFONO_OK = "Teléfono creado";
    public static final String MENSAJE_ALTA_TELEFONO_ERROR_DATOS = "El tipo y/o número están vacíos";
    public static final String MENSAJE_ALTA_TELEFONO_ERROR = "Error al guardar el teléfono";
    
    public static final String MENSAJE_BORRAR_TELEFONO_INEXISTENTE = "No se especificó un teléfono";
    public static final String MENSAJE_BORRAR_TELEFONO_OK = "Teléfono borrado";
    
    public static final String MENSAJE_MODIFICAR_TELEFONO_INEXISTENTE = "No se especificó un teléfono, tipo y/o número";
    public static final String MENSAJE_MODIFICAR_TELEFONO_OK = "Teléfono modificado";
    
    
    private int idPersona;
    private String apellidos;
    private String nombres;
    private LocalDate fechaNacimiento;
    private String direccion;
    private String correo;
        
    /** Constructor 
     * @param apellidos apellidos de la persona
     * @param nombres nombres de la persona
     * @param fechaNacimiento fecha de nacimiento de la persona
     * @param direccion dirección de la persona
     * @param email email de la persona
    */
    public Persona(String apellidos, String nombres, LocalDate fechaNacimiento, String direccion, String email) {
        this(0, apellidos, nombres, fechaNacimiento, direccion, email);
    }

    /** Constructor 
     * @param idPersona identificador de la persona
     * @param apellidos apellidos de la persona
     * @param nombres nombres de la persona
     * @param fechaNacimiento fecha de nacimiento de la persona
     * @param direccion dirección de la persona
     * @param email email de la persona
    */    
    public Persona (int idPersona, String apellidos, String nombres, LocalDate fechaNacimiento, String direccion, String email) {
        this.idPersona = idPersona;
        this.apellidos = apellidos;
        this.nombres = nombres;
        this.fechaNacimiento = fechaNacimiento;
        this.direccion = direccion;
        this.correo = email;        
    }

    /** Devuelve el identificador de la persona
     * @return int con el identificador de la persona
    */        
    public int verID() {
        return this.idPersona;
    }

    /** Asigna el identificador a la persona
     * @param idPersona identificador de la persona
    */            
    public void asignarID(int idPersona) {
        this.idPersona = idPersona;
    }

    /** Devuelve los apellidos de la persona
     * @return String con los apellidos de la persona
    */              
    public String verApellidos() {
        return this.apellidos;
    }

    /** Asigna los apellidos a la persona
     * @param apellidos apellidos de la persona
    */                
    public void asignarApellidos(String apellidos) {
        this.apellidos = apellidos;
    }

    /** Devuelve los nombres de la persona
     * @return String con los nombres de la persona
    */            
    public String verNombres() {
        return this.nombres;
    }

    /** Asigna los nombres a la persona
     * @param nombres nombres de la persona
    */                
    public void asignarNombres(String nombres) {
        this.nombres = nombres;
    }

    /** Devuelve la fecha de nacimiento de la persona
     * @return LocalDate con la fecha de nacimiento de la persona
    */            
    public LocalDate verFecNac() {
        return this.fechaNacimiento;
    }

    /** Asigna la fecha de nacimiento a la persona
     * @param fechaNacimiento fecha de nacimiento de la persona
    */                
    public void asignarFecNac(LocalDate fechaNacimiento) {
        this.fechaNacimiento = fechaNacimiento;
    }

    /** Devuelve la dirección de la persona
     * @return String con la dirección de la persona
    */            
    public String verDireccion() {
        return this.direccion;
    }

    /** Asigna la dirección a la persona
     * @param direccion dirección de la persona
    */                
    public void asignarDireccion(String direccion) {
        this.direccion = direccion;
    }

    /** Devuelve el correo de la persona
     * @return String con el correo de la persona
    */            
    public String verCorreo() {
        return this.correo;
    }

    /** Asigna el correo a la persona
     * @param correo correo de la persona
    */                
    public void asignarCorreo(String correo) {
        this.correo = correo;
    }
        
    /**
     * Devuelve la lista de teléfonos de la persona especificada
     * Si no se especifica una persona, o se produce un error en la BD, devuelve null
     * Si la persona especificada no tiene teléfonos, se devuelve una lista vacía
     * @return List<Telefono>  - lista con los teléfonos de la persona especificada
     * @see Telefono
    */                                                        
    public List<Telefono> verTelefonos()  {
        List<Telefono> telefonos = new ArrayList<>();
        CallableStatement cstmt = null;
        try {
            Connection cnn = Conexion.verConnection();
            String sql = "{call asp_dame_telefonos(?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setInt(1, this.idPersona);
            ResultSet rs = cstmt.executeQuery();
            while (rs.next()) {
                int idTelefono = rs.getInt("IDTelefono");
                String tipo = rs.getString("Tipo");
                TipoTelefono tt = null;
                if (tipo.equalsIgnoreCase("Particular"))
                    tt = TipoTelefono.PARTICULAR;
                else if (tipo.equalsIgnoreCase("Celular"))
                    tt = TipoTelefono.CELULAR;
                else if (tipo.equalsIgnoreCase("Trabajo"))
                    tt = TipoTelefono.TRABAJO;
                else
                    tt = TipoTelefono.OTRO;
                String numero = rs.getString("Numero");
                Telefono unTelefono = new Telefono(this.idPersona, idTelefono, tt, numero);
                telefonos.add(unTelefono);
            }
            rs.close();
            return telefonos;
        }
        catch(Exception e) {
            return null;
        }
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }            
    }
    
    /**
     * Agrega un teléfono a la persona
     * @param tipo tipo de teléfono a agregar
     * @param numero número de teléfono a agregar
     * @return String  - cadena con el resultado de la operación
     * @see Telefono
     * @see Persona
    */                            
    public String agregarTelefono (String tipo, String numero) {
        Connection cnn = Conexion.verConnection();
        CallableStatement cstmt = null;
        try {
            String sql = "{call asp_alta_telefono(?, ?, ?, ?)}";
            cstmt = cnn.prepareCall(sql);
            cstmt.setInt(1, this.idPersona);
            cstmt.setString(2, tipo);
            cstmt.setString(3, numero);
            cstmt.registerOutParameter(4, java.sql.Types.VARCHAR);
            cstmt.executeUpdate();
            return cstmt.getString(4);
        }
        catch (Exception e) {
            return e.getMessage();
        } 
        finally {
            if (cstmt != null) {
                try {
                    cstmt.close();
                }
                catch(SQLException e) {

                }
            }
        }            
    }

    /**
     * Borra un teléfono a la persona
     * @param telefono telefono a borrar
     * @return String  - cadena con el resultado de la operación
     * @see Telefono
    */                    
    public String borrarTelefono (Telefono telefono) {
        if (telefono != null) {
            Connection cnn = Conexion.verConnection();
            CallableStatement cstmt = null;
            try {
                String sql = "{call asp_borrar_telefono(?, ?, ?)}";
                cstmt = cnn.prepareCall(sql);
                cstmt.setInt(1, this.idPersona);
                cstmt.setInt(2, telefono.verIDTelefono());
                cstmt.registerOutParameter(3, java.sql.Types.VARCHAR); 
                cstmt.executeUpdate();
                return cstmt.getString(3);
            }
            catch (Exception e)      {
               return e.getMessage();
            }
            finally {
                if (cstmt != null) {
                    try {
                        cstmt.close();
                    }
                    catch(SQLException e) {

                    }
                }
            }            
        }
        else {
            return MENSAJE_TELEFONO_ERROR_CONEXION;
        }            
    }    
    
    /**
     * Modifica un teléfono de la persona
     * @param telefono telefono a modificar
     * @param tipo tipo del teléfono modificado
     * @param numero número del teléfono modificado
     * @return String  - cadena con el resultado de la operación
     * @see Telefono
    */                            
    public String modificarTelefono (Telefono telefono, String tipo, String numero) {
        if ((telefono != null) && (tipo != null) && (numero != null)) {
            Connection cnn = Conexion.verConnection();
            CallableStatement cstmt = null;
            try {
                String sql = "{call asp_modificar_telefono(?, ?, ?, ?, ?)}";
                cstmt = cnn.prepareCall(sql);
                cstmt.setInt(1, this.idPersona);
                cstmt.setInt(2, telefono.verIDTelefono());
                cstmt.setString(3, tipo);
                cstmt.setString(4, numero);
                cstmt.registerOutParameter(5, java.sql.Types.VARCHAR); 
                cstmt.executeUpdate();
                return cstmt.getString(5);
            }
            catch (Exception e)      {
               return e.getMessage();
            }
            finally {
                if (cstmt != null) {
                    try {
                        cstmt.close();
                    }
                    catch(SQLException e) {

                    }
                }
            }
        }
        else {
            return MENSAJE_TELEFONO_ERROR_CONEXION;
        }
    }        
    
}
